/// \file a429_cdev.h
/// \brief инфраструктура для работы с символьными устройствами каналов pci устройства
#ifndef A429_CDEV_H
#define A429_CDEV_H

#include "a429_pci.h"

/// \brief имя шаблона класса символьного устройтсва
#define A429_CDEV_CLASS                  "a429dev"
/// \brief кол-во символов в имени
#define COUNT_NAME_SYMBOLS                  32
/// \brief невалидное значение MAJOR
#define BAD_DEV_MAJOR                       -1

/// \brief размер блока ДМА (128 байт)
#define SIZE_RAW_DMA_BLOCK                  128
/// \brief мегабайт в байтах (размер кольцевого буфера для одного канала)
#define MEGABYTE                            1024*1024

/// \brief The a429_cdev_data struct информация о символьном устройстве (канал ДПК)
struct a429_cdev_data {
    struct device*              device;                         ///< девайс
    struct cdev                 cdev;                           ///< символьное устройство
    uint32_t                    dev_major;                      ///< major индекс
    int                         index;                          ///< индекс в массиве символьных устройств
    uint32_t                    numCh;                          ///< номер канала (minor индекс)
    struct A429_DEV*            drv;                            ///< ссылка на драйвер
    char                        devname[COUNT_NAME_SYMBOLS];    ///< имя устройства
    spinlock_t                  spinlockCh;                     ///< спинлок для канала
    uint32_t*                   dma_wr;                         ///< указазатель для записи дма канала
    uint32_t*                   dma_rd;                         ///< указазатель для чтения дма канала
    unsigned int                numRkIn;                        ///< номер ВХ РК для реализации запрос/готовность
};

/// \brief The a429_cdev_private struct приватная информация о символьном устройстве, которая прикомпоновывается к символьному устройству
struct a429_cdev_private {
    u_int                       index;                          ///< индекс в массиве символьных устройств
};

///
/// \brief clear_cdev_array сброс символьных устройств
void clear_cdev_array(void);

/// \brief get_info_channel получить информацию о канале
/// \param data информация о символьном устройстве (канале)
/// \return информация о канале
T_INFO_CHANNEL get_info_channel(struct a429_cdev_data* data);

/// \brief create_char_devs создание символьных устройств для каналов
/// \param driver_priv информационная структура об устройстве
/// \return код ошибки
int create_char_devs(struct A429_DEV* driver_priv);

/// \brief create_char_devs удаление символьных устройств для каналов
/// \param driver_priv информационная структура об устройстве
/// \return код ошибки
int destroy_char_devs(struct A429_DEV* driver_priv);

/// \brief getCdevPtr получить ссылку на информацию о символном устройстве (канале)
/// \param priv внутренняя структура символьного устройства
/// \return информационная структура о символьном устройстве
struct a429_cdev_data* getCdevPtr(struct a429_cdev_private *priv);

/// \brief getCdevPtrByIndex получить ссылку на информацию о символном устройстве (канале)
/// \param index индекс символьного устройства канала
/// \return информационная структура о символьном устройстве
struct a429_cdev_data* getCdevPtrByIndex(int index);

/// \remark ПРИМЕЧАНИЕ:
/// дескрипторы символьных устройств каналов (приёмников и передатчиков) нумеруются с 0 (сначала - приёмники, потом передатчики)

#endif // A429_CDEV_H
