/// \file a429_info.h
/// \brief общая информация и парметры модуля ядра

#ifndef A429_INFO_H
#define A429_INFO_H

/// \remark информация о дате и версии
/// \brief старший номер версии
#define DRIVER_MAJOR_VER                2
/// \brief младший номер версии
#define DRIVER_MINOR_VER                3
/// \brief день создания
#define DRIVER_DATE_DAY                 3
/// \brief месяц создания
#define DRIVER_DATE_MONTH               11
/// \brief год создания
#define DRIVER_DATE_YEAR                23
/// \brief закодированная дата и версия
///  31..28 - major_ver; 27..24 - minor_ver; 23..16 - day; 15..8 - month; 7..0 - year;
#define DRIVER_DATE_N_VERSION           ((DRIVER_MAJOR_VER & 0xF) << 28) | ((DRIVER_MINOR_VER & 0xF) << 24) | ((DRIVER_DATE_DAY & 0xFF) << 16) | ((DRIVER_DATE_MONTH & 0xFF) << 8) | (DRIVER_DATE_YEAR & 0xFF)

/// \brief имя драйвера
#define DRIVER_NAME                     "A429_DRV"
/// \brief версия драйвера
#define DRIVER_VERSION                  "2.3"
/// \brief описание драйвера
#define DRIVER_DESCRIPTION              "a429 pci card driver"
/// \brief права драйвера
#define DRIVER_COPYRIGHT                "'NOVOMAR' LLC"
/// \brief лицензия драйвера
#define DRIVER_LICENSE                  "GPL"
/// \brief шаблон имени pci-устройства
#define PCI_DEVICE_NAME_TEMPLATE        "a429_dev_"

#define VENDOR_ID_NOVOMAR               0xA203      ///< вендор
#define SUBVENDOR_ID_NOVOMAR            0xA203      ///< субвендор

#define DEVICE_ID_PCIE_A429             0x9477      ///< идентификатор поддерживаемого устройства A429 PCIe
#define DEVICE_ID_MPCIE_A429            0x9478      ///< идентификатор поддерживаемого устройства A429 mPCIe
#define DEVICE_ID_XMC_A429              0x9481      ///< идентификатор поддерживаемого устройства A429 XMC

#define DEVICE_CLASS_CODE_A429          0x78000     ///< class code поддерживаемого устройства

#define DEVICE_REV_ID_A429_UD88         0x01        ///< ревизия поддерживаемого устройства UD88
#define DEVICE_REV_ID_A429_UD84         0x11        ///< ревизия поддерживаемого устройства UD84
#define DEVICE_REV_ID_A429_UD44         0x21        ///< ревизия поддерживаемого устройства UD44
#define DEVICE_REV_ID_A429_UD66         0x31        ///< ревизия поддерживаемого устройства UD66
#define DEVICE_REV_ID_A429_UD22         0x41        ///< ревизия поддерживаемого устройства UD22

#define MAX_COUNT_BOARDS                32          ///< максимально допустимое количество pci-устройств одновременно
#define MAX_RECEIVE_CHANNELS            16           ///< максимально допустимое количество каналов на приём
#define MAX_TRANSMIT_CHANNELS           16           ///< максимально допустимое количество каналов на передачу
/// \brief максимально возможное кол-во каналов в плате
#define MAX_CHANNELS                    (MAX_RECEIVE_CHANNELS + MAX_TRANSMIT_CHANNELS)

#define DMA_SIZE                        1048576     ///< размер дма буфера

#endif

