/// \file a429_log.h
/// \brief простой логгер в dmesg

#ifndef A429_LOG_H
#define A429_LOG_H

#include <linux/kernel.h>
#include "a429_info.h"

///
/// \brief __log_func функция логирования в dmesg
/// \param lvl уровень
/// \param fmt строка
///
static void __log_func(const char* lvl, const char* fmt, ...)
{
    va_list args;
    va_start(args, fmt);
    printk("%s [%s]: ", DRIVER_NAME, lvl);
    vprintk(fmt, args);
    va_end(args);
}

//#define DEBUG_CONFIGURATION

#ifdef DEBUG_CONFIGURATION

#ifndef LOG_LEVEL_INFO
/// \brief уровень "сообщение"
#define LOG_LEVEL_INFO "INFO"
#endif

#ifndef LOG_LEVEL_WARNING
/// \brief уровень "предупреждение"
#define LOG_LEVEL_WARNING "WARNING"
#endif

#ifndef LOG_INFO
/// \brief логирование сообщения
#define LOG_INFO(fmt, args...) \
		__log_func(LOG_LEVEL_INFO, fmt, ## args)
#endif

#ifndef LOG_WARNING
/// \brief логирование предупреждения
#define LOG_WARNING(fmt, args...) \
		__log_func(LOG_LEVEL_WARNING, fmt, ## args)
#endif

#else // DEBUG_CONFIGURATION

#ifndef LOG_INFO
/// \brief логирование сообщения
#define LOG_INFO(fmt, args...)
#endif

#ifndef LOG_WARNING
/// \brief логирование предупреждения
#define LOG_WARNING(fmt, args...)
#endif

#endif // DEBUG_CONFIGURATION

#ifndef LOG_LEVEL_ERROR
/// \brief уровень "ошибка"
#define LOG_LEVEL_ERROR "ERROR"
#endif

#ifndef LOG_ERROR
/// \brief логирование ошибки
#define LOG_ERROR(fmt, args...) \
        __log_func(LOG_LEVEL_ERROR, fmt, ## args)
#endif

#ifndef LOG_LEVEL_RELEASE_INFO
/// \brief уровень "инфо всегда"
#define LOG_LEVEL_RELEASE_INFO "RELEASE_INFO"
#endif

#ifndef LOG_RELEASE_INFO
/// \brief логирование ошибки
#define LOG_RELEASE_INFO(fmt, args...) \
        __log_func(LOG_LEVEL_RELEASE_INFO, fmt, ## args)
#endif

#endif
