/// \file a429_pci.h
/// \brief инфраструктура для работы с pci устройством
#ifndef A429_PCI_H
#define A429_PCI_H

#include "a429_info.h"
#include "a429_cmd.h"

#include <linux/types.h>
#include <linux/init.h>
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/vmalloc.h>
#include <linux/interrupt.h>
#include <linux/fs.h>
#include <linux/pci.h>
#include <linux/cdev.h>
#include <linux/kthread.h>
#include <linux/completion.h>
#include <linux/sched.h>
#include <linux/semaphore.h>
#include <linux/delay.h>
#include <linux/errno.h>
#include <linux/uaccess.h>
#include <linux/mutex.h>
#include <asm/msr.h>
#include <linux/spinlock.h>
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/signal.h>
#include <linux/sched.h>
#include <linux/version.h>

///
/// \brief The DMA_INFO struct информация для взаимодействия с ДМА
///
struct DMA_INFO {
    size_t                  dma_size;                       ///< размер примаппленной области ДМА
    dma_addr_t              dma_paddr;                      ///< указатель на память ДМА
    uint8_t*                dma_vaddr;                      ///< виртуальный указатель намять ДМА
};

///
/// \brief The A429_DEV struct базовая структура для работы с pci устройтсвом
///
struct A429_DEV { 
    u8 __iomem*             pbar;                           ///< указатель на примаппленную память устройства
    uint64_t                barlen;                         ///< размер примаппленной памяти
    int                     bar;                            ///< адрес реcурса ввода/вывода
    struct pci_dev*         pdevice;                        ///< указатель на базовую pci_dev
    u_int                   unit;                           ///< номер pci устройства в массиве pdevs
    char                    devname[30];                    ///< имя pci устройства
    uint16_t                vendorid;                       ///< идентификатор вендора
    uint16_t                deviceid;                       ///< идентификатор устройства
    uint16_t                revisionid;                     ///< идентификатор ревизии
    struct class*           a429cdev_class;                 ///< класс символьного устройства канала
    char                    a429cdev_name[30];              ///< имя класса символьного устройства каналаv
    int                     index_cdev[MAX_RECEIVE_CHANNELS + MAX_TRANSMIT_CHANNELS];   ///< индексы символьных устройств каналов pci-платы
    int                     curCountReceiveChannels;        ///< фактическое количество каналов в плате на приём
    int                     curCountTransmitChannels;       ///< фактическое количество каналов в плате на передачу
    struct DMA_INFO         dma_info;                       ///< информация о ДМА
    spinlock_t              spinlockIrq;                    ///< спинлок для синхронизации с прерываниями
    unsigned long           spinlockIrqFlags;               ///< флаги спинлока для синхронизации с прерываниями
    T_TRACKING_INT          trackingInt;                    ///< управление слежением за прерываниями
    T_CHECK_TRACKING_INT    checkTrackingInt;               ///< информация о слежении за прерываниями
};

#endif // A429_PCI_H
